% Finding the best solution with Tikhonov-Regularization
% First, a mean rate is estimated.
% Only the ratio criterion is used for estimating the optimal alpha.


function [R_out,C_out,alpha_opt] = calculate_con_rates_lin_sys_tichonov_mean_rate_2(C_water,bnd_cond,alpha_ticho,l_0,l_1,l_2,z_c,C_c,omega,D_total,beta,phi)


%  -------- scaling the data ------------
C_scal = max(C_c);
 
C_c = C_c /C_scal;
C_water = C_water / C_scal;

bnd_cond.C_z_min = bnd_cond.C_z_min / C_scal;
bnd_cond.C_z_max = bnd_cond.C_z_max / C_scal;
% -------------------------------------


dis_hor_sep
disp('                Starting Tichonov - Regularization');
disp(' ');


% ------- some pre-calculations ------
z_c_red = z_c(2:end-1);
C_c_red = C_c(2:end-1);
N_red = length(z_c_red);

C_c_red = make_column_vector(C_c_red);

delta_z = z_c(2) - z_c(1);
% -------------------------------------------


% ----- Calculate the neccessary matrices ------
[A,e,F,d] = calculate_diff_operator_matrix_aequi_dist_grid_variable_coeff(z_c,D_total,omega,beta,phi,C_water,bnd_cond);
A_ad = A';

[B,L0,L1,L2] = calculate_B_matrix(l_0,l_1,l_2,N_red,z_c_red);
disp('ready with Tichonov - Matrices: A and B');
disp(' ');

C_hat = C_c_red + e;
% ----------------------------------------------


% --------- finding the mean rate and respective concentration ------
[R_mean,C_mean] = find_mean_rate(A,C_hat);
C_tilde = C_hat - C_mean;
% --------------------------------------------------------------------

% -------- Estimating the best alpha Parameter -----------------------
for k = 1:length(alpha_ticho)
    
    k;
    alpha = alpha_ticho(k);
    
	% ------- finding the tichonov solution --------  	
    R_tilde = inv(A_ad*A + alpha*B) * A_ad * C_tilde;	
	%disp('ready with rate coefficients');
	% ----------------------------------------------
	
	% ------- constructing the final solution -------
	[R_tich_c_m(k,:),C_tich_c_m(k,:)] = determine_con_and_rate(delta_z,R_tilde,A,e,R_mean,C_mean,bnd_cond);
	% -----------------------------------------------
	
	% -------- calculate Tichonov-optimal parameter function -------
    quot_crit(k) = optimal_ticho_parameter_quotientenkriterium(R_tilde,C_tilde,B,A,alpha);
	% --------------------------------------------------------------
    
end


% ------ finding the minima of the criteria functions --        
[mini,index,found] = find_local_minimum_with_smallest_x(-quot_crit,alpha_ticho);
found
if found == 0
    [mini,index] = min(-quot_crit);  % no local minimum
end
disp('finding optimal alpha with ratio criterion');        
        
plot_parameter_kriteria(alpha_ticho,quot_crit,index);

alpha_opt = alpha_ticho(index);
disp(['optimal alpha for Tichonov - Regularization:']);
alpha_opt

R_out = R_tich_c_m(index,:);
C_out = C_tich_c_m(index,:);
% ------------------------------------------------------------------------

% -------- re-scaling the data ------------
C_out = C_out * C_scal;
R_out = R_out *  C_scal;
% -------------------------------------



return
%-------end of main code-------------------------------



% =========================================================================

%  Helping routines

% =========================================================================



%------------------------------------------
function [R_c,C_c] = determine_con_and_rate(delta_z,R_tilde,A,e,R_mean,C_mean,bnd_cond)
% R_c : total rate from linear system on computational grid
% C_c : total nutrient concentration from linear system on computational grid


% ------ reading the boundary conditions structure ----
[C_z_min,C_z_max,type_z_min,type_z_max] = read_bnd_cond_structure(bnd_cond);
% -----------------------------------------------------

% ------ finding total nutrient concentration  ---
C_tilde = A*R_tilde;
C_hat_r = C_tilde + C_mean;
C_r = C_hat_r - e;

% top boundary condition
if  type_z_min == 1   % Dirichlet conditions
        bnd_z_min = C_z_min;
      
elseif type_z_min == 2 % von Neumann conditions
        m_z_min = C_z_min;
        C_2 = C_r(1);
        C_3 = C_r(2);
        bnd_z_min = 4/3*C_2 - 1/3*C_3 - 2/3*delta_z*m_z_min;
end

% bottom boundary condition
if  type_z_max == 1   % Dirichlet conditions
        bnd_z_max = C_z_max;
      
elseif type_z_max == 2 % von Neumann conditions
        m_z_max = C_z_max;
        C_Nminus1 = C_r(end);
        C_Nminus2 = C_r(end-1);
        bnd_z_max = 4/3*C_Nminus1 - 1/3*C_Nminus2 + 2/3*delta_z*m_z_max;
end


C_c = [bnd_z_min; C_r; bnd_z_max];
% -------------------------------------------------



% -- finding total rate --------------------------
R_red = R_tilde + R_mean;
R_c = [R_red(1) ; R_red ; R_red(end)];
% ------------------------------------------------

% ------ making row-vectors ----------------
R_c = make_row_vector(R_c);
C_c = make_row_vector(C_c);
% ------------------------------------------

return
%---------------------------------------------------




% --------------------------------------------------
function quot = optimal_ticho_parameter_quotientenkriterium(R_tilde,C_tilde,B,A,alpha)

R_tilde = make_column_vector(R_tilde);

% ----- determine derivative -----
H = inv(A'*A + alpha*B);
d = -B*R_tilde;
ablei = H*d;
% --------------------------------

% ------ determine residuum -----
res = A*R_tilde - C_tilde;
% -------------------------------

quot = norm( A*(alpha*ablei) - res ) / norm(res);

return
% --------------------------------------------------




% ----------- calculate B-Matrix -------------------
function [B,L0,L1,L2] = calculate_B_matrix(lam_0,lam_1,lam_2,N_red,z_c)
% Matrix B for sympathy functional

delta_z = z_c(2) - z_c(1);

% ------ L0 -----------------
L0 = eye(N_red);
% ---------------------------

% ------ L1 / 1st Derivative -----------------
N_L1 = N_red - 1;
L1 = zeros(N_L1,N_red);

for i = 1:N_L1
    L1(i,i) = -1;
    L1(i,i+1) = 1;
end

L1 = 1/delta_z * L1;
% ------------------------------------------------

% ------ L2 / 2nd Derivative -----------------
N_L2 = N_red - 2;
L2 = zeros(N_L2,N_red);

for i = 1:N_L2
    L2(i,i)   = 1;
    L2(i,i+1) = -2;
    L2(i,i+2) = 1;
end

L2 = 1/(delta_z)^2 * L2;
% ------------------------------------------------

% ------- B -------------------------------------
B = lam_0*L0'*L0 + lam_1*L1'*L1 + lam_2*L2'*L2;
% -----------------------------------------------
return

% --------------------------------------------------



% --------------------------------------------------
% ------ Finding the mean rate 
function [R_mean,C_mean] = find_mean_rate(A,C_hat)

N_red = length(C_hat);


E = ones(N_red,1);
M = A*E;

R_c = 1/(M'*M)*(M'*C_hat);
R_mean = R_c * ones(N_red,1);


C_mean = A*R_mean;
return
% ---------------------------------------------------


% ---------------------------------------------------
function plot_parameter_kriteria(alpha_ticho,quot_crit,index)

figure(3); clf; hold on;

set(gca,'Fontsize',14);
plot(alpha_ticho,-quot_crit,'k-','Linewidth',2);
plot(alpha_ticho(index),-quot_crit(index),'ko','Linewidth',2);
%set(gca,'xscale','log','yscale','log');
set(gca,'xscale','log');
ylabel('T(\alpha)');
xlabel('\alpha');
box on;
title('ratio criterion for finding the optimal \alpha');
axis tight;
% ---------------------------------------------------

pause(1);

return

