function [A,e,Diff_op,d] = calculate_diff_operator_matrix_aequi_dist_grid_variable_coeff(z,D,omega,beta,por,C_water,bnd_cond)
% Function to construct the discrete inverse Differential Operator A for Berg problem
% on an aequidistant grid in z direction.
% It is A = inv(Diff_op).
%
% The coefficient functions por, D, omega and beta are z-dependend.
%
% For the first derivative term, a blended scheme according to Boudreau is
% used. For the second derivative term, a central difference scheme is used.

N_grid = length(z);
N = N_grid - 2;  % Dimension of Matrix: N = N_grid - 2

% --------- reading the bnd_con structure -----
[C_z_min,C_z_max,type_z_min,type_z_max] = read_bnd_cond_structure(bnd_cond);
% ---------------------------------------------

% ----- make row vectors --------------
z = make_row_vector(z);
D = make_row_vector(D);
omega = make_row_vector(omega);
beta = make_row_vector(beta);
por = make_row_vector(por);
% -------------------------------------

% ------- determine grid spacings -------
delta_z = z(2) - z(1);
% ---------------------------------------

% ------ construct coefficient vectors r(z) and s(z) -----
omega2 = omega.*por;
D2 = D.*por;
beta2 = beta.*por;

D2_abl = abl_1(D2,z);
omega2_abl = abl_1(omega2,z);

r = omega2_abl + beta2;
s = omega2 - D2_abl;
% -----------------------------------------------------

% ----- construct blending factor sigma ---------------
for i = 1:N_grid    
    if s(i) == 0
        sig(i) = 0;
    else
        eta = s(i)*delta_z / ( 2 * D2(i) );
        sig(i) = coth(eta) - 1/eta;        
    end
end
% -------------------------------------------------------

% -------- construct the matrix coefficients ------------
aa = -(1+sig).*s/(2*delta_z) - D2/(delta_z)^2;
bb = r + s.*sig/delta_z + 2*D2/(delta_z)^2;
cc = (1-sig).*s/(2*delta_z) - D2/(delta_z)^2;
% -------------------------------------------------------

% ----- Construction of the forward Diff_op -------------------------
Diff_op = zeros(N,N);

for i=2:N-1
    Diff_op(i,i-1) = aa(i+1);
    Diff_op(i,i)   = bb(i+1);
    Diff_op(i,i+1) = cc(i+1);
end

% first line
if  type_z_min == 1   % Dirichlet conditions
        disp('Using Dirichlet boundary conditions for top boundary')
        Diff_op(1,1) = bb(2);
        Diff_op(1,2) = cc(2);
     
elseif type_z_min == 2 % von Neumann conditions
        disp('Using von Neumann boundary conditions for top boundary')
        Diff_op(1,1) = bb(2) + 4/3*aa(2);
        Diff_op(1,2) = cc(2) - 1/3*aa(2);
end

%N th line
if  type_z_max == 1   % Dirichlet conditions
    disp('Using Dirichlet boundary conditions for bottom boundary')
    Diff_op(N,N-1) = aa(N_grid-1);
    Diff_op(N,N) = bb(N_grid-1);
        
elseif type_z_max == 2 % von Neumann conditions
    disp('Using von Neuman boundary conditions for bottom boundary')
    Diff_op(N,N-1) = aa(N_grid-1) - 1/3*cc(N_grid-1);
    Diff_op(N,N) = bb(N_grid-1) + 4/3*cc(N_grid-1);

end
% -------------------------------------------------------


% ------ construction of A -------------------------
%Diff_op
A = inv(Diff_op);
% -------------------------------------------------

% ------- Constructing e -----------------
d = -C_water*beta2(2:N_grid-1)';

% first line of d
if  type_z_min == 1   % Dirichlet conditions
    d(1) = d(1) + aa(2)*C_z_min;  
    
elseif type_z_min == 2 % von Neumann conditions
    % remember, m_z_min=C_z_min
    m_z_min = C_z_min; 
    d(1) = d(1) - 2/3*delta_z*aa(2)*m_z_min; 
end

% last line of d
if  type_z_max == 1   % Dirichlet conditions
    d(N) = d(N) + cc(N_grid-1)*C_z_max;
    
elseif type_z_max == 2 % von Neumann conditions
    % remember, m_z_max=C_z_max
    m_z_max = C_z_max;     
    d(N) = d(N) + 2/3*delta_z*cc(N_grid-1)*m_z_max;
end

e = A*d;
% ----------------------------------------

return
% -----------------------------------------



% =========================================================================
%        Helping functions
% =========================================================================



% ------ make row vector -------
function z = make_row_vector(z)

% function to guaranty that z is a row vector afterwards
if size(z,1) > 1
    z = z';
end

return