% =========================================================================
%                     REC_V3_with_GUI                                    
% 
% This matlab program tries to calculate consumption and
% production rates from measured nutrient concentration profile
% on the basis of a Tikhonov regularization.
%
% This program comes with no graphical user interface (GUI).
%
% I hope it is free of fundamental errors.
% If not, please inform me: lettmann@icbm.de / k.lettmann@gmx.de
% =========================================================================

function REC_tikhonov_func(setup_name,N_c,C_water,lambda,alpha_min,alpha_max,N_alpha,bnd_cond)

global z_c R_out

% ========== Input - Parameter ======================
% setup_name =  Name of setup and name of the data folder
% N_c = Number of computational grid points
% C_water =  Nutrient concentration in water column (only important for irrigation)
% 
% 
% % parameters for tikhonov regularisation
% lambda =  'smoothing' parameter lambda
% alpha_min =  lowest alpha value for Tikhonov regularisation and ratio criterion ( actually log_10(alpha_min) )
% alpha_max =  largest alpha value for Tikhonov regularisation and ratio criterion ( actually log_10(alpha_max) )
% N_alpha =  Number of ratio criterion evaluations in the alpha interval, to find the minimum
% 
% 
% % setting the boundary conditions for the nutrient concentration
% bnd_cond.type_z_min =  type of boundary condition at the top: 1: for concentration / 2: for derivative
% bnd_cond.C_z_min =  value of nutrient concentration or derivative at top
% 
% bnd_cond.type_z_max =  type of boundary condition at the bottom: 1: for concentration / 2: for derivative
% bnd_cond.C_z_max =  value of nutrient concentration or derivative at bottom
% ============ End of parameter input ======================================



% =========================================================================
%    The program starts from here on. 
%    No changes to the code should be made below these lines.
% =========================================================================


% =========== some pre - operations   ======================================
l_0 = 1;
l_1 = lambda;
l_2 = lambda;
alpha_ticho = logspace(alpha_min,alpha_max,N_alpha);
% =========================================================================

% =========  reading the Input Data =======================================
[z_data,C_data,z_c,C_c,omega,D_total,beta,phi,error_read_input] = read_input_data_func(setup_name,N_c);
% =========================================================================


% ========= Determine Concentration and Rates =============================
[R_out,C_out,alpha_opt] = calculate_con_rates_lin_sys_tichonov_mean_rate_2(C_water,bnd_cond,alpha_ticho,l_0,l_1,l_2,z_c,C_c,omega,D_total,beta,phi);
% =========================================================================


% ======= graphical output of the results ===========
figure(4); clf; hold on;

subplot(1,2,1); hold on;
set(gca,'Fontsize',15);
plot(C_data,z_data,'ko','Linewidth',2);
plot(C_out,z_c,'g','Linewidth',2);
set(gca,'YDir','reverse');
title('C [\muM]');
ylabel('z [cm]');
axis tight;
box on;

subplot(1,2,2); hold on;
set(gca,'Fontsize',15);
plot(R_out,z_c,'r','Linewidth',2);
set(gca,'YDir','reverse');
title('Rates [nmol / (cm^3 s)]');
ylabel('z [cm]');
axis tight;
box on;


% printing the figure to the setup folder
current_dir = pwd;
if isunix == 1
    % We are working on a Unix/Linux or Macintosh system
    out_name = [current_dir,'/',setup_name,'/',setup_name,'_output_figure.jpg'];
else
    % We are working on a Windows system    
    out_name = [current_dir,'\',setup_name,'\',setup_name,'_output_figure.jpg'];
end
print(gcf,'-djpeg','-r300',out_name)
% =====================================================


% ========= write to output file ======================
write_results(setup_name,R_out,C_out,z_c);
% =====================================================


% ========= calculate fluxes about the interval interfaces =====
write_fluxes(setup_name,C_out,z_c,omega,D_total,phi)
% ===============================================================

return




% =======================================================
%          Helping routines 
% =======================================================


function write_results(setup_name,R_out,C_out,z_out)

dis_hor_sep
disp('                Writing Results to output file');
disp(' ');

current_dir = pwd;
if (isunix == 1) 
    setup_name = [current_dir,'/',setup_name,'/',setup_name];
else
    setup_name = [current_dir,'\',setup_name,'\',setup_name];
end

disp('Output File:');
output_file = [setup_name,'_output.txt']

A(:,1) = make_column_vector(z_out);
A(:,2) = make_column_vector(C_out);
A(:,3) = make_column_vector(R_out);

fid = fopen(output_file, 'wt');
fprintf(fid, 'z [cm]       C [muM]     Rates [nmol/(cm^3 s)]\n');
for i=1:size(A,1)
    fprintf(fid, '%6.3e %6.3e %6.3e\n', A(i,:));
end
fclose(fid);

return




% ==================================================================
function write_fluxes(setup_name,C_out,z_c,omega,D_total,phi)
% this function calcualtes the total diffusive and advective fluxes
% across the interfaces of the considered depth interval


dis_hor_sep
disp('                Writing diffusive and advective fluxes to output file');
disp(' ');

current_dir = pwd;
if (isunix == 1) 
    setup_name = [current_dir,'/',setup_name,'/',setup_name];
else
    setup_name = [current_dir,'\',setup_name,'\',setup_name];
end

disp('Flux-Output File:');
output_file = [setup_name,'_output_fluxes.txt']


% ---- calculate diffusive fluxes ---------
C_out_abl = abl_1(C_out,z_c);

flux_diff_top = - phi(1)*D_total(1)*C_out_abl(1);
flux_diff_bottom = - phi(end)*D_total(end)*C_out_abl(end);
% ------------------------------------------

% ----- calculate advective fluxes ---------
flux_adv_top = phi(1)*omega(1)*C_out(1);
flux_adv_bottom = phi(end)*omega(end)*C_out(end);
% ------------------------------------------

% ------- total flux -----------------------
flux_total_top = flux_diff_top + flux_adv_top;
flux_total_bottom = flux_diff_bottom + flux_adv_bottom;
% --------------------------------------------

% ------ write the data to output file -------
fid = fopen(output_file, 'wt');
fprintf(fid, 'Output file for flux calculations.  \n');
fprintf(fid, 'A flux is positive, if it is directed into positive z-direction (from top to bottom).  \n');
fprintf(fid, '  \n');
fprintf(fid, 'The diffusive flux is defined as: -phi * (D_effective + D_bio) * dC/dz \n');
fprintf(fid, 'The advective flux is defined as: phi * omega * C \n');
fprintf(fid, 'The total flux is the sum of the diffusive and the advective  flux. \n');
fprintf(fid, '  \n');
fprintf(fid, 'All fluxes are given in:    [cm * muM / s] \n');
fprintf(fid, 'Or in related units, if not the standard unit system is used. \n');
fprintf(fid, '  \n');
fprintf(fid, '         diffusive flux    advective flux    total flux \n');
fprintf(fid, '------------------------------------------------------------- \n');
fprintf(fid, ['top    : ',num2str(flux_diff_top,'%7.4e'),'       ',num2str(flux_adv_top,'%7.4e'),'       ',num2str(flux_total_top,'%7.4e'),'\n']);
fprintf(fid, ['bottom : ',num2str(flux_diff_bottom,'%7.4e'),'       ',num2str(flux_adv_bottom,'%7.4e'),'       ',num2str(flux_total_bottom,'%7.4e'),'\n']);

fclose(fid);

return